@@ -13,18 +13,19 @@
 ##############################################################################
 """Pagelet mixin classes
 
-$Id: browser.py 80190 2007-09-27 08:22:54Z zagy $
+$Id: browser.py 97315 2009-02-26 16:08:59Z nadako $
 """
 __docformat__ = 'restructuredtext'
 
 import zope.interface
 import zope.component
-from zope.formlib import form
+from zope.formlib import form # TODO: get rid of hard dependency on zope.formlib
 from zope.publisher import browser
 
 from z3c.template.interfaces import ILayoutTemplate, IContentTemplate
 from z3c.pagelet import interfaces
 
+REDIRECT_STATUS_CODES = (301, 302, 303)
 
 # default pagelet base implementation
 class BrowserPagelet(browser.BrowserPage):
@@ -41,17 +42,28 @@ class BrowserPagelet(browser.BrowserPage):
     def render(self):
         # render content template
         if self.template is None:
-            template = zope.component.getMultiAdapter(
-                (self, self.request), IContentTemplate)
+            template = zope.component.queryMultiAdapter(
+                (self, self.request, self.context), IContentTemplate)
+            if template is None:
+                template = zope.component.getMultiAdapter(
+                    (self, self.request), IContentTemplate)
             return template(self)
         return self.template()
 
     def __call__(self):
         """Calls update and returns the layout template which calls render."""
         self.update()
+        
+        if self.request.response.getStatus() in REDIRECT_STATUS_CODES:
+            # don't bother rendering when redirecting
+            return ''
+        
         if self.layout is None:
-            layout = zope.component.getMultiAdapter(
-                (self, self.request), ILayoutTemplate)
+            layout = zope.component.queryMultiAdapter(
+                (self, self.request, self.context), ILayoutTemplate)
+            if layout is None:
+                layout = zope.component.getMultiAdapter(
+                    (self, self.request), ILayoutTemplate)
             return layout(self)
         return self.layout()
 
@@ -78,8 +90,11 @@ class PageletForm(form.FormBase, BrowserPagelet):
                 self.resetForm()
                 self.form_reset = False
             if self.template is None:
-                template = zope.component.getMultiAdapter(
-                    (self, self.request), IContentTemplate)
+                template = zope.component.queryMultiAdapter(
+                    (self, self.request, self.context), IContentTemplate)
+                if template is None:
+                    template = zope.component.getMultiAdapter(
+                        (self, self.request), IContentTemplate)
                 self.form_result = template(self)
             else:
                 self.form_result = self.template()
@@ -98,8 +113,11 @@ class PageletAddForm(PageletForm, form.AddFormBase):
             return ""
         # render content template
         if self.template is None:
-            template = zope.component.getMultiAdapter(
-                (self, self.request), IContentTemplate)
+            template = zope.component.queryMultiAdapter(
+                (self, self.request, self.context), IContentTemplate)
+            if template is None:
+                template = zope.component.getMultiAdapter(
+                    (self, self.request), IContentTemplate)
             return template(self)
         return self.template()
 
